"""
GroupSelector contains parent classes for all group selectors. A GroupSelector
is used at the stochastic engine's runtime to select groups upon which a
move will be applied. Therefore it has become possible to fully customize the
selection of groups of atoms and to choose  when and how frequently a group
can be chosen to perform a move upon.

.. inheritance-diagram:: fullrmc.Core.GroupSelector
    :parts: 1
"""
# standard libraries imports
from __future__ import print_function
import inspect

# external libraries imports
import numpy as np

# fullrmc imports
from ..Globals import INT_TYPE, FLOAT_TYPE, LOGGER
from ..Globals import str, long, unicode, bytes, basestring, range, xrange, maxint
from ..Core.Collection import ListenerBase, is_integer
from ..Core.Group import Group
from ..Core.MoveGenerator import PathGenerator


class GroupSelector(object):
    """
    Group selector is the parent class that selects groups to perform
    moves at stochastic engine's runtime.

    :Parameters:
        #. engine (None, fullrmc.Engine): Selector's stochastic engine instance.
    """

    def __init__(self, engine=None):
        # init GroupSelector
        super(GroupSelector, self).__init__()
        # set engine
        self.set_engine(engine)

    def __getstate__(self):
        state = {}
        for k in self.__dict__:
            if k == '_GroupSelector__engine':
                state[k] = None
            else:
                state[k] = self.__dict__[k]
        return state

    def _runtime_initialize(self):
        """ This method is called by the engine at the runtime to initialize
        the group selector if needed.
        """
        pass

    @property
    def engine(self):
        """ Stochastic engine's instance."""
        return self.__engine

    @property
    def refine(self):
        """ Get refine flag value. It will always return False because
        refine is a property of RecursiveGroupSelector instances only. """
        return False

    @property
    def explore(self):
        """ Get explore flag value. It will always return False because
        explore is a property of RecursiveGroupSelector instances only. """
        return False

    @property
    def willSelect(self):
        """ Get whether next step a new selection is occur or still the same
        group is going to be selected again. It will always return True
        because recurrence is a property of RecursiveGroupSelector instances
        only. """
        return True

    @property
    def willRecur(self):
        """ Get whether next step the same group will be returned. It will
        always return False because this is a property of
        RecursiveGroupSelector instances only. """
        return False

    @property
    def willRefine(self):
        """ Get whether selection is recurring and refine flag is True. It
        will always return False because recurrence is a property of
        RecursiveGroupSelector instances only. """
        return False

    @property
    def willExplore(self):
        """ Get whether selection is recurring and explore flag is True.
        It will always return False because recurrence is a property of
        RecursiveGroupSelector instances only. """
        return False

    @property
    def isNewSelection(self):
        """ Get whether the last step a new selection was made. It will
        always return True because recurrence is a property of
        RecursiveGroupSelector instances only. """
        return True

    @property
    def isRecurring(self):
        """ Get whether the last step the same group was returned. It will
        always return False because this is a property of
        RecursiveGroupSelector instances only."""
        return False

    @property
    def isRefining(self):
        """ Get whether selection is recurring and refine flag is True. It
        will always return False because recurrence is a property of
        RecursiveGroupSelector instances only. """
        return False

    @property
    def isExploring(self):
        """ Get whether selection is recurring and explore flag is True. It
        will always return False because recurrence is a property of
        RecursiveGroupSelector instances only. """
        return False

    def set_engine(self, engine):
        """
        Set selector's stochastic engine instance.

        :Parameters:
            #. engine (None, fullrmc.Engine): Selector's stochastic engine.
        """
        if engine is not None:
            from fullrmc.Engine import Engine
            assert isinstance(engine, Engine), LOGGER.error("engine must be None or fullrmc Engine instance")
        self.__engine = engine

    def select_index(self):
        """
        This method must be overloaded in every GroupSelector sub-class

        :Returns:
            #. index (integer): the selected group index in engine groups list.
        """
        raise Exception(LOGGER.error("%s '%s' method must be overloaded"%(self.__class__.__name__,inspect.stack()[0][3])))

    def move_accepted(self, index):
        """
        This method is called by the stochastic engine when a move generated
        on a group is accepted. This method is empty must be overloaded when
        needed.

        :Parameters:
            #. index (integer): the selected group index in engine groups list.
        """
        pass

    def move_rejected(self, index):
        """
        This method is called by the stochastic engine when a move generated
        on a group is rejected. This method is empty must be overloaded when
        needed.

        :Parameters:
            #. index (integer): the selected group index in engine groups list.
        """
        pass


class RecursiveGroupSelector(GroupSelector):
    """
    Recursive selector is the only selector that can use the recursive
    property on a selection. It is used as a wrapper around a GroupSelector
    instance.

    :Parameters:
        #. selector (fullrmc.Core.GroupSelector.GroupSelector): The selector
           instance to wrap.
        #. recur (integer): Set number of times to recur. It must be a
           positive integer.
        #. override (boolean): Override temporary recur value.
           recur value will be overridden only when selected group move
           generator is a PathGenerator instance. In this particular case,
           recur value will be temporary changed to the number of moves
           stored in the PathGenerator. If selected group move generator
           is not a PathGenerator instance, recur value will take back its
           original value.
        #. refine (boolean): Its an engine flag that is used to refine
           the position of a group until recurrence expires and a new group
           is selected. Refinement is done by applying moves upon the selected
           group always from its initial position at the time it was selected
           until recurrence expires, then the best position is kept.
        #. explore (boolean): Its an engine flag that is used to make a group
           explore the space around it until recurrence expires and a new
           group is selected. Exploring is done by applying moves upon
           the selected group starting from its initial position and evolving
           in a trajectory like way until recurrence expires, then the best
           position is kept.

    **NB**: refine and explore flags can't both be set to True at the same time.
    When this happens refine flag gets automatically switched to False.
    The usage of those flags is very important because they allow groups of
    atoms to go out of local minima in the energy surface. The way traditional
    reverse mote carlo works is by minimizing the total energy of the system
    (error) using gradient descent method. Using of those flags allows the
    system to go up hill  in the energy surface searching for other lower
    minimas, while always conserving the lowest energy state found and not
    changing the system structure until a better structure with smaller error
    is found.

    .. raw:: html

        <p>
        The following video compares the Reverse Monte Carlo traditional fitting mode
        with fullrmc's recursive selection one with explore flag set to True.
        From a potential point of view, exploring allows to cross forbidden unlikely
        energy barriers and going out of local minimas.
        </p>
        <iframe width= "560" height="315"
        src="https://www.youtube.com/embed/24Rd2EZ2vVo?rel=0"
        frameborder="0" allowfullscreen>
        </iframe>

        <p>
        The following video is an example of refining the position of a molecule using
        RecursiveGroupSelector and setting refine flag to True.
        The molecule is always refined from its original position towards a new one generated by the move generator.
        </p>
        <iframe width="560" height="315"
        src="https://www.youtube.com/embed/tFYHOD00j_o?rel=0"
        frameborder="0" allowfullscreen>
        </iframe>

        <p>
        The following video is an example of exploring the space of a molecule using
        RecursiveGroupSelector and setting explore flag to True.
        The molecule explores the allowed space by wandering via its move generator and only
        moves enhancing the structure are stored.
        </p>
        <iframe width="560" height="315"
        src="https://www.youtube.com/embed/wafgTfHNpvs?rel=0"
        frameborder="0" allowfullscreen>
        </iframe>

    .. code-block:: python

        # import fullrmc modules
        from fullrmc.Engine import Engine
        from fullrmc.Core.GroupSelector import RecursiveGroupSelector

        # create engine
        ENGINE = Engine(path='my_engine.rmc')

        # set pdb file
        ENGINE.set_pdb('system.pdb')

        # Add constraints ...
        # Re-define groups if needed ...
        # Re-define groups selector if needed ...

        ##### Wrap engine group selector with a recursive group selector. #####
        # create recursive group selector. Recurrence is set to 20 with explore flag set to True.
        RGS = RecursiveGroupSelector(ENGINE.groupSelector, recur=20, refine=False, explore=True)
        ENGINE.set_group_selector(RGS)
    """

    def __init__(self, selector, recur=10, override=True, refine=False, explore=False):
        # set selector instance
        assert isinstance(selector, GroupSelector),LOGGER.error("selector must be a fullrmc GroupSelector instance")
        assert not isinstance(selector, RecursiveGroupSelector),LOGGER.error("selector can't not be a RecursiveGroupSelector instance")
        self.__selector = selector
        # initialize selector
        super(RecursiveGroupSelector, self).__init__(engine=self.__selector.engine)
        # change engine selector to
        if self.__selector.engine is not None:
            self.__selector.engine.set_group_selector(self)
        # set recur value
        self.set_recur(recur)
        # set override
        self.set_override(override)
        # set refine
        self.set_refine(refine)
        # set explore
        self.set_explore(explore)
        # initialize
        self.__initialize_selector__()

    def __initialize_selector__(self):
        # initialize last selected index
        self.__lastSelectedIndex = None
        self.__position = self.__recur

    def _runtime_initialize(self):
        """ This method is called by the engine at the runtime to initialize
        the group selector if needed.
        """
        self.__selector._runtime_initialize()

    @property
    def selector(self):
        """ The wrapped selector instance."""
        return self.__selector

    @property
    def lastSelectedIndex(self):
        """ The last selected group index."""
        return self.__lastSelectedIndex

    @property
    def willSelect(self):
        """ Get whether next step a new selection is occur or still the same
        group is going to be selected again. """
        return self.__position >= self.__recur

    @property
    def willRecur(self):
        """ Get whether next step the same group will be returned. """
        return not self.willSelect

    @property
    def willRefine(self):
        """ Get whether next step the same group will be returned and
        refine flag is True."""
        return self.isRecurring and self.__refine

    @property
    def willExplore(self):
        """ Get whether next step the same group will be returned and
        explore flag is True."""
        return self.isRecurring and self.__explore

    @property
    def isNewSelection(self):
        """ Get whether this last step a new selection was made. """
        return self.__position <= 1

    @property
    def isRecurring(self):
        """ Get whether this last step the same group was returned. """
        return not self.isNewSelection

    @property
    def isRefining(self):
        """ Get whether this last step the same group was returned and
        refine flag is True."""
        return self.isRecurring and self.__refine

    @property
    def isExploring(self):
        """ Get whether this last step the same group was returned and
        explore flag is True."""
        return self.isRecurring and self.__explore

    @property
    def override(self):
        """ Override flag value. """
        return self.__override

    @property
    def refine(self):
        """ Refine flag value. """
        return self.__refine

    @property
    def explore(self):
        """ Explore flag value. """
        return self.__explore

    @property
    def currentRecur(self):
        """ The current recur value which is selected group dependant
        when override flag is True."""
        return self.__recur

    @property
    def recur(self):
        """
        The current recur value. The set recur value can change during engine
        runtime if override flag is True. To get the recur value as set by
        set_recur method recurAsSet must be used.
        """
        return self.__recur

    @property
    def recurAsSet(self):
        """ Get recur value as set but set_recur method."""
        return self.__recurAsSet

    @property
    def position(self):
        """ Get the position of the selector in the path."""
        return self.__position

    @property
    def engine(self):
        """ Get the wrapped selector engine instance."""
        return self.__selector.engine

    def set_engine(self, engine):
        """
        Sets the wrapped selector stochastic engine instance.

        :Parameters:
            #. engine (None, fullrmc.Engine): The selector stochastic engine.
        """
        self.__selector.set_engine(engine)

    def set_recur(self, recur):
        """
        Sets the recur value.

        :Parameters:
            #. recur (integer): Set the recur value.
               It must be a positive integer.
        """
        assert is_integer(recur), LOGGER.error("recur must be an integer")
        recur = INT_TYPE(recur)
        assert recur>=0, LOGGER.error("recur must be positive")
        self.__recur      = recur
        self.__recurAsSet = recur

    def set_override(self, override):
        """
        Select override value.

        :Parameters:
            #. override (boolean): Override selector recur value only when
               selected group move generator is a PathGenerator instance.
               Overridden recur value is temporary and totally selected group
               dependant. If selected group move generator is not a
               PathGenerator instance, recur value will take back selector's
               recur value.
        """
        assert isinstance(override, bool), LOGGER.error("override must be a boolean")
        self.__override = override

    def set_refine(self, refine):
        """
        Select override value.

        :Parameters:
            #. refine (boolean): Its an engine flag that is used to refine the
               position of a group until recurrence expires and a new group is
               selected. Refinement is done by applying moves upon the selected
               group always from its initial position at the time it was
               selected until recurrence expires, then the best position
               is kept.
        """
        assert isinstance(refine, bool), LOGGER.error("refine must be a boolean")
        self.__refine = refine

    def set_explore(self, explore):
        """
        Select override value.

        :Parameters:
            #. explore (boolean): Its an engine flag that is used to make a
               group explore the space around it until recurrence expires and
               a new group is selected. Exploring is done by applying moves upon
               the selected group starting from its initial position and
               evolving in a trajectory like way until recurrence expires,
               then the best position is kept.
        """
        assert isinstance(explore, bool), LOGGER.error("explore must be a boolean")
        self.__explore = explore
        if self.__refine and self.__explore:
            LOGGER.log("argument fixed", "refine and explore flags are not allowed both True. Conflict is resolved by setting refine flag to False")
            self.__refine = False

    def select_index(self):
        """
        Select new index.

        :Returns:
            #. index (integer): the selected group index in engine groups list.
        """
        # select new group
        if self.willSelect:
            self.__lastSelectedIndex = self.__selector.select_index()
            # reset count
            self.__position = 0
            # reset recur
            if self.__override:
                groupGenerator = self.engine.groups[self.__lastSelectedIndex].moveGenerator
                if isinstance(groupGenerator, PathGenerator):
                    self.__recur = len(groupGenerator.path)
            else:
                self.__recur = self.__recurAsSet
        # move position by 1
        self.__position += 1
        # return selection
        return self.__lastSelectedIndex



"""
AtomicCoordinationConstraints contains classes for all constraints related
to coordination number in spherical shells around atoms.

.. inheritance-diagram:: fullrmc.Constraints.AtomicCoordinationConstraints
    :parts: 1

"""
# standard libraries imports
from __future__ import print_function
import copy

# external libraries imports
import numpy as np

# fullrmc imports
from ..Globals import INT_TYPE, FLOAT_TYPE, PI, LOGGER
from ..Globals import str, long, unicode, bytes, basestring, range, xrange, maxint
from ..Core.Collection import is_number, is_integer, raise_if_collected, reset_if_collected_out_of_date
from ..Core.Collection import get_caller_frames
from ..Core.Constraint import SingularConstraint, RigidConstraint
from ..Core.atomic_coordination import all_atoms_coord_number_coords, multi_atoms_coord_number_coords


class AtomicCoordinationNumberConstraint(RigidConstraint, SingularConstraint):
    """
    It's a rigid constraint that controls the coordination number of atoms.

    .. raw:: html

        <iframe width="560" height="315"
        src="https://www.youtube.com/embed/R8t-_XwizOI?rel=0"
        frameborder="0" allowfullscreen>
        </iframe>


    +----------------------------------------------------------------------+
    |.. figure:: atomic_coordination_number_constraint_plot_method.png     |
    |   :width: 530px                                                      |
    |   :height: 400px                                                     |
    |   :align: left                                                       |
    +----------------------------------------------------------------------+


    :Parameters:
        #. rejectProbability (Number): rejecting probability of all steps
           where standardError increases. It must be between 0 and 1 where 1
           means rejecting all steps where standardError increases and 0 means
           accepting all steps regardless whether standardError increases or
           not.


    .. code-block:: python

        # import fullrmc modules
        from fullrmc.Engine import Engine
        from fullrmc.Constraints.AtomicCoordinationConstraints import AtomicCoordinationNumberConstraint

        # create engine
        ENGINE = Engine(path='my_engine.rmc')

        # set pdb file
        ENGINE.set_pdb('system.pdb')

        # create and add constraint
        ACNC = AtomicCoordinationNumberConstraint()
        ENGINE.add_constraints(ACNC)

        # create definition
        ACNC.set_coordination_number_definition( [ ('Al','Cl',1.5, 2.5, 2, 2),
                                                   ('Al','S', 2.5, 3.0, 2, 2)] )

    """
    def __init__(self, rejectProbability=1):
        # initialize constraint
        RigidConstraint.__init__(self, rejectProbability=rejectProbability)
        # set atomsColletors data keys
        self._atomsCollector.set_data_keys( ('coresIndexes', 'shellsIndexes',
                                             'asCoreDefIdxs', 'inShellDefIdxs') )
        # initialize data
        self.__initialize_constraint_data()
        # set coordination number definition
        self.set_coordination_number_definition(None)
        # set computation cost
        self.set_computation_cost(5.0)
        # set frame data
        FRAME_DATA = [d for d in self.FRAME_DATA]
        FRAME_DATA.extend(['_AtomicCoordinationNumberConstraint__coordNumDef',
                           '_AtomicCoordinationNumberConstraint__coresIndexes',
                           '_AtomicCoordinationNumberConstraint__numberOfCores',
                           '_AtomicCoordinationNumberConstraint__shellsIndexes',
                           '_AtomicCoordinationNumberConstraint__lowerShells',
                           '_AtomicCoordinationNumberConstraint__upperShells',
                           '_AtomicCoordinationNumberConstraint__minAtoms',
                           '_AtomicCoordinationNumberConstraint__maxAtoms',
                           '_AtomicCoordinationNumberConstraint__coordNumData',
                           '_AtomicCoordinationNumberConstraint__weights',
                           '_AtomicCoordinationNumberConstraint__asCoreDefIdxs',
                           '_AtomicCoordinationNumberConstraint__inShellDefIdxs',] )
        RUNTIME_DATA = [d for d in self.RUNTIME_DATA]
        RUNTIME_DATA.extend( ['_AtomicCoordinationNumberConstraint__coordNumData',
                              '_AtomicCoordinationNumberConstraint__coresIndexes',
                              '_AtomicCoordinationNumberConstraint__shellsIndexes',
                              '_AtomicCoordinationNumberConstraint__asCoreDefIdxs',
                              '_AtomicCoordinationNumberConstraint__inShellDefIdxs'] )
        object.__setattr__(self, 'FRAME_DATA',   tuple(FRAME_DATA)   )
        object.__setattr__(self, 'RUNTIME_DATA', tuple(RUNTIME_DATA) )

    def __initialize_constraint_data(self):
        # set definition
        self.__coordNumDef = None
        # the following is the parsing of defined shells
        self.__coresIndexes  = []
        self.__numberOfCores = []
        self.__shellsIndexes = []
        self.__lowerShells   = []
        self.__upperShells   = []
        self.__minAtoms      = []
        self.__maxAtoms      = []
        # upon computing constraint data, those values must be divided by len( self.__coresIndexes[i] )
        self.__coordNumData = []
        self.__weights      = []
        # atoms to cores and shells pointers
        self.__asCoreDefIdxs  = []
        self.__inShellDefIdxs = []
        # no need to dump to repository because all of those attributes will be written
        # at the point of setting the definition.

    def _on_collector_reset(self):
        pass

    @property
    def coordNumDef(self):
        """Copy of coordination number definition dictionary"""
        return copy.deepcopy( self.__coordNumDef )

    @property
    def coresIndexes(self):
        """List of coordination number core atoms index array."""
        return self.__coresIndexes

    @property
    def shellsIndexes(self):
        """List of coordination number shell atoms index array."""
        return self.__shellsIndexes

    @property
    def lowerShells(self):
        """Array of lower shells distance."""
        return self.__lowerShells

    @property
    def upperShells(self):
        """Array of upper shells distance. """
        return self.__upperShells

    @property
    def minAtoms(self):
        """Array of minimum number of atoms in a shell."""
        return self.__minAtoms

    @property
    def maxAtoms(self):
        """Array of maximum number of atoms in a shell. """
        return self.__maxAtoms

    @property
    def weights(self):
        """Shells weight which count in the computation of standard error."""
        return self.__weights

    @property
    def data(self):
        """Coordination number constraint data."""
        return self.__coordNumData

    @property
    def asCoreDefIdxs(self):
        """List of arrays where each element is pointing to a coordination
        number definition where the atom is a core."""
        return self.__asCoreDefIdxs

    @property
    def inShellDefIdxs(self):
        """List of arrays where each element is pointing to a coordination
        number definition where the atom is in a shell."""
        return self.__inShellDefIdxs

    def listen(self, message, argument=None):
        """
        listen to any message sent from the Broadcaster.

        :Parameters:
            #. message (object): Any python object to send to constraint's
               listen method.
            #. argument (object): Any type of argument to pass to the listeners.
        """
        if message in("engine set", "update molecules indexes"):
            self.set_coordination_number_definition(self.__coordNumDef)
            # reset constraint is called in set_coordination_number_definition
        elif message in("update boundary conditions",):
            self.reset_constraint()

    #@raise_if_collected
    def set_coordination_number_definition(self, coordNumDef):
        """
        Set the coordination number definition.

        :Parameters:
            #. coordNumDef (None, list, tuple): Coordination number definition.
               It must be None, list or tuple where every element is a list or
               a tuple of exactly 6 items and an optional 7th item for weight.

               #. core atoms: Can be any of the following:

                  * string: indicating atomic element.
                  * dictionary: Key as atomic attribute among (element, name)
                    and value is the attribute value.
                  * list, tuple, set, numpy.ndarray: core atoms index.

               #. in shell atoms: Can be any of the following:

                  * string: indicating atomic element.
                  * dictionary: Key as atomic attribute among (element, name)
                    and value is the attribute value.
                  * list, tuple, set, numpy.ndarray: in shell atoms index

               #. Lower distance limit of the coordination shell.
               #. Upper distance limit of the coordination shell.
               #. :math:`N_{min}` : minimum number of neighbours in the
                  shell.
               #. :math:`N_{max}` : maximum number of neighbours in the
                  shell.
               #. :math:`W_{i}` : weight contribution to the standard error,
                  this is optional, if not given it is set automatically to 1.0.

               ::

                   e.g. [ ('Ti','Ti', 2.5, 3.5, 5, 7.1, 1), ('Ni','Ti', 2.2, 3.1, 7.2, 9.7, 100), ...]
                        [ ({'element':'Ti'},'Ti', 2.5, 3.5, 5, 7.1, 0.1), ...]
                        [ ({'name':'au'},'Au', 2.5, 3.5, 4.1, 6.3), ...]
                        [ ({'name':'Ni'},{'element':'Ti'}, 2.2, 3.1, 7, 9), ...]
                        [ ('Ti',range(100,500), 2.2, 3.1, 7, 9), ...]
                        [ ([0,10,11,15,1000],{'name':'Ti'}, 2.2, 3.1, 7, 9, 5), ...]

        """
        if self.engine is None:
            self.__coordNumDef = coordNumDef
            return
        elif coordNumDef is None:
            coordNumDef = []
        ########## check definitions, create coordination number data ##########
        self.__initialize_constraint_data()
        ALL_NAMES       = self.engine.get_original_data("allNames")
        NAMES           = self.engine.get_original_data("names")
        ALL_ELEMENTS    = self.engine.get_original_data("allElements")
        ELEMENTS        = self.engine.get_original_data("elements")
        NUMBER_OF_ATOMS = self.engine.get_original_data("numberOfAtoms")
        for CNDef in coordNumDef:
            assert isinstance(CNDef, (list, tuple)), LOGGER.error("coordNumDef item must be a list or a tuple")
            if len(CNDef) == 6:
                coreDef, shellDef, lowerShell, upperShell, minCN, maxCN = CNDef
                weight = 1.0
            elif len(CNDef) == 7:
                coreDef, shellDef, lowerShell, upperShell, minCN, maxCN, weight = CNDef
            else:
                raise LOGGER.error("coordNumDef item must have 6 or 7 items")
            # core definition
            if isinstance(coreDef, basestring):
                coreDef = str(coreDef)
                assert coreDef in ELEMENTS, LOGGER.error("core atom definition '%s' is not a valid element"%coreDef)
                coreIndexes = [idx for idx, el in enumerate(ALL_ELEMENTS) if el==coreDef]
            elif isinstance(coreDef, dict):
                assert len(coreDef) == 1, LOGGER.error("core atom definition dictionary must be of length 1")
                key, value = list(coreDef)[0], list(coreDef.values())[0]
                if key is "name":
                    assert value in NAMES, LOGGER.error("core atom definition '%s' is not a valid name"%coreDef)
                    coreIndexes = [idx for idx, el in enumerate(ALL_NAMES) if el==coreDef]
                elif key is "element":
                    assert value in ELEMENTS, LOGGER.error("core atom definition '%s' is not a valid element"%coreDef)
                    coreIndexes = [idx for idx, el in enumerate(ALL_ELEMENTS) if el==coreDef]
                else:
                    raise LOGGER.error("core atom definition dictionary key must be either 'name' or 'element'")
            elif isinstance(coreDef, (list, tuple, set, np.ndarray)):
                coreIndexes = []
                if isinstance(coreDef, np.ndarray):
                    assert len(coreDef.shape)==1, LOGGER.error("core atom definition numpy.ndarray must be 1D")
                for c in coreDef:
                    assert is_integer(c), LOGGER.error("core atom definition index must be integer")
                    c = INT_TYPE(c)
                    assert c>=0, LOGGER.error("core atom definition index must be >=0")
                    assert c<NUMBER_OF_ATOMS, LOGGER.error("core atom definition index must be smaler than number of atoms in system")
                    coreIndexes.append(c)
            # shell definition
            if isinstance(shellDef, basestring):
                shellDef = str(shellDef)
                assert shellDef in ELEMENTS, LOGGER.error("core atom definition '%s' is not a valid element"%shellDef)
                shellIndexes = [idx for idx, el in enumerate(ALL_ELEMENTS) if el==shellDef]
            elif isinstance(shellDef, dict):
                assert len(shellDef) == 1, LOGGER.error("core atom definition dictionary must be of length 1")
                key, value = list(shellDef)[0], list(shellDef.values())[0]
                if key is "name":
                    assert value in NAMES, LOGGER.error("core atom definition '%s' is not a valid name"%shellDef)
                    shellIndexes = [idx for idx, el in enumerate(ALL_NAMES) if el==shellDef]
                elif key is "element":
                    assert value in ELEMENTS, LOGGER.error("core atom definition '%s' is not a valid element"%shellDef)
                    shellIndexes = [idx for idx, el in enumerate(ALL_ELEMENTS) if el==shellDef]
                else:
                    raise LOGGER.error("core atom definition dictionary key must be either 'name' or 'element'")
            elif isinstance(shellDef, (list, tuple, set, np.ndarray)):
                shellIndexes = []
                if isinstance(shellDef, np.ndarray):
                    assert len(shellDef.shape)==1, LOGGER.error("core atom definition numpy.ndarray must be 1D")
                for c in shellDef:
                    assert is_integer(c), LOGGER.error("core atom definition index must be integer")
                    c = INT_TYPE(c)
                    assert c>=0, LOGGER.error("core atom definition index must be >=0")
                    assert c<NUMBER_OF_ATOMS, LOGGER.error("core atom definition index must be smaler than number of atoms in system")
                    shellIndexes.append(c)
            # lower and upper shells definition
            assert is_number(lowerShell), LOGGER.error("Coordination number lower shell '%s' must be a number."%lowerShell)
            lowerShell = FLOAT_TYPE(lowerShell)
            assert lowerShell>=0, LOGGER.error("Coordination number lower shell '%s' must be a positive."%lowerShell)
            assert is_number(upperShell), LOGGER.error("Coordination number upper shell '%s' must be a number."%key)
            upperShell = FLOAT_TYPE(upperShell)
            assert upperShell>lowerShell, LOGGER.error("Coordination number lower shell '%s' must be smaller than upper shell %s"%(lowerShell,upperShell))
            # minimum and maximum number of atoms definitions
            assert is_number(minCN), LOGGER.error("Coordination number minimum atoms '%s' must be a number."%minCN)
            minCN = FLOAT_TYPE(minCN)
            assert minCN>=0, LOGGER.error("Coordination number minimim atoms '%s' must be >=0."%minCN)
            assert is_number(maxCN), LOGGER.error("Coordination number maximum atoms '%s' must be a number."%key)
            maxCN = FLOAT_TYPE(maxCN)
            assert maxCN>=minCN, LOGGER.error("Coordination number minimum atoms '%s' must be smaller than maximum atoms %s"%(minCN,maxCN))
            # check weight
            assert is_number(weight), LOGGER.error("Coordination number weight '%s' must be a number."%weight)
            weight = FLOAT_TYPE(weight)
            assert weight>0, LOGGER.error("Coordination number weight '%s' must be >0."%weight)
            # append coordination number data
            self.__coresIndexes.append( sorted(set(coreIndexes)) )
            self.__shellsIndexes.append( sorted(set(shellIndexes)) )
            self.__lowerShells.append( lowerShell )
            self.__upperShells.append( upperShell )
            self.__minAtoms.append( minCN )
            self.__maxAtoms.append( maxCN )
            #self.__coordNumData.append( FLOAT_TYPE(0) )
            self.__coordNumData.append( None )
            self.__weights.append( weight )
        ########## set asCoreDefIdxs and inShellDefIdxs points ##########
        for _ in xrange(NUMBER_OF_ATOMS):
            self.__asCoreDefIdxs.append( [] )
            self.__inShellDefIdxs.append( [] )
        for defIdx, indexes in enumerate(self.__coresIndexes):
            self.__coresIndexes[defIdx] = np.array( indexes, dtype=INT_TYPE )
            for atIdx in indexes:
                self.__asCoreDefIdxs[atIdx].append( defIdx )
        for defIdx, indexes in enumerate(self.__shellsIndexes):
            self.__shellsIndexes[defIdx] = np.array( indexes, dtype=INT_TYPE )
            for atIdx in indexes:
                self.__inShellDefIdxs[atIdx].append( defIdx )
        for atIdx in xrange(NUMBER_OF_ATOMS):
            self.__asCoreDefIdxs[atIdx]  = np.array( self.__asCoreDefIdxs[atIdx], dtype=INT_TYPE )
            self.__inShellDefIdxs[atIdx] = np.array( self.__inShellDefIdxs[atIdx], dtype=INT_TYPE )
        # set all to arrays
        #self.__coordNumData  = np.array( self.__coordNumData, dtype=FLOAT_TYPE )
        self.__weights       = np.array( self.__weights, dtype=FLOAT_TYPE )
        self.__numberOfCores = np.array( [len(idxs) for idxs in self.__coresIndexes], dtype=FLOAT_TYPE )
        # set definition
        self.__coordNumDef = coordNumDef
        # dump to repository
        self._dump_to_repository({'_AtomicCoordinationNumberConstraint__coordNumDef'  :self.__coordNumDef,
                                  '_AtomicCoordinationNumberConstraint__coordNumData' :self.__coordNumData,
                                  '_AtomicCoordinationNumberConstraint__weights'      :self.__weights,
                                  '_AtomicCoordinationNumberConstraint__numberOfCores':self.__numberOfCores,
                                  '_AtomicCoordinationNumberConstraint__coresIndexes' :self.__coresIndexes,
                                  '_AtomicCoordinationNumberConstraint__shellsIndexes':self.__shellsIndexes,
                                  '_AtomicCoordinationNumberConstraint__lowerShells'  :self.__lowerShells,
                                  '_AtomicCoordinationNumberConstraint__upperShells'  :self.__upperShells,
                                  '_AtomicCoordinationNumberConstraint__minAtoms'     :self.__minAtoms,
                                  '_AtomicCoordinationNumberConstraint__maxAtoms'     :self.__maxAtoms})
        # reset constraint
        self.reset_constraint() # ADDED 2017-JAN-08

    def compute_standard_error(self, data):
        """
        Compute the standard error (StdErr) of data not satisfying constraint's
        conditions.

        .. math::
            StdErr = \\sum \\limits_{i}^{S} Dev_{i}


        .. math::
            Dev_{i}=\\begin{cases}
              W_{i}*( N_{min,i}-\\overline{CN_{i}} ), & \\text{if $\\overline{CN_{i}}<N_{min,i}$}.\\\\
              W_{i}*( \\overline{CN_{i}}-N_{max,i} ), & \\text{if $\\overline{CN_{i}}>N_{max,i}$}.\\\\
              0                  , & \\text{if $N_{min,i}<=\\overline{CN_{i}}<=N_{max,i}$}
            \\end{cases}


        Where:\n
        :math:`S`                  is the total number of defined coordination number shells. \n
        :math:`W_{i}`              is the defined weight of coordination number shell i. \n
        :math:`Dev_{i}`            is the standard deviation of the coordination number in shell definition i. \n
        :math:`\\overline{CN_{i}}` is the mean coordination number value in shell definition i. \n
        :math:`N_{min,i}`          is the defined minimum number of neighbours in shell definition i. \n
        :math:`N_{max,i}`          is the defined maximum number of neighbours in shell definition i. \n


        :Parameters:
            #. data (numpy.array): The constraint value data to compute
               standardError.

        :Returns:
            #. standardError (number): The calculated standardError of the
               constraint.
        """
        coordNum = data/self.__numberOfCores
        StdErr   = 0.
        for idx, cn in enumerate( coordNum ):
            if cn < self.__minAtoms[idx]:
                StdErr += self.__weights[idx]*(self.__minAtoms[idx]-cn)
            elif cn > self.__maxAtoms[idx]:
                StdErr += self.__weights[idx]*(cn-self.__maxAtoms[idx])
        return StdErr

    # THIS NEEDS TO BE FORMATTED HUMAN READABLE RETURN 2017-07-29
    def get_constraint_value(self):
        """
        Get constraint's data.

        :Returns:
            #. data (numpy.array): The constraint value data
        """
        return self.data

    @reset_if_collected_out_of_date
    def compute_data(self):
        """ Compute constraint's data. """
        self.__coordNumData = np.array( [FLOAT_TYPE(0) for _ in self.__coordNumData], dtype=FLOAT_TYPE )
        all_atoms_coord_number_coords(boxCoords      = self.engine.boxCoordinates,
                                      basis          = self.engine.basisVectors,
                                      isPBC          = self.engine.isPBC,
                                      coresIndexes   = self.__coresIndexes,
                                      shellsIndexes  = self.__shellsIndexes,
                                      lowerShells    = self.__lowerShells,
                                      upperShells    = self.__upperShells,
                                      asCoreDefIdxs  = self.__asCoreDefIdxs,
                                      inShellDefIdxs = self.__inShellDefIdxs,
                                      coordNumData   = self.__coordNumData,
                                      ncores         = self.engine._runtime_ncores)
        self.__coordNumData /= FLOAT_TYPE(2.)
        # update data
        self.set_data( self.__coordNumData )
        self.set_active_atoms_data_before_move(None)
        self.set_active_atoms_data_after_move(None)
        # set standardError
        stdErr = self.compute_standard_error(data = self.__coordNumData)
        self.set_standard_error(stdErr)
        # set original data
        if self.originalData is None:
            self._set_original_data(self.data)

    def compute_before_move(self, realIndexes, relativeIndexes):
        """
        Compute constraint's data before move is executed.

        :Parameters:
            #. realIndexes (numpy.ndarray): Not used here.
            #. relativeIndexes (numpy.ndarray): Group atoms relative index
               the move will be applied to.
        """
        beforeMoveData = np.zeros(self.__coordNumData.shape, dtype=self.__coordNumData.dtype)
        multi_atoms_coord_number_coords( indexes        = relativeIndexes,
                                         boxCoords      = self.engine.boxCoordinates,
                                         basis          = self.engine.basisVectors,
                                         isPBC          = self.engine.isPBC,
                                         coresIndexes   = self.__coresIndexes,
                                         shellsIndexes  = self.__shellsIndexes,
                                         lowerShells    = self.__lowerShells,
                                         upperShells    = self.__upperShells,
                                         asCoreDefIdxs  = self.__asCoreDefIdxs,
                                         inShellDefIdxs = self.__inShellDefIdxs,
                                         coordNumData   = beforeMoveData,
                                         ncores         = self.engine._runtime_ncores)
        # set active atoms data before move
        self.set_active_atoms_data_before_move( beforeMoveData )
        self.set_active_atoms_data_after_move(None)

    def compute_after_move(self, realIndexes, relativeIndexes, movedBoxCoordinates):
        """
        Compute constraint's data after move is executed.

        :Parameters:
            #. realIndexes (numpy.ndarray): Not used here.
            #. relativeIndexes (numpy.ndarray): Group atoms relative index
               the move will be applied to.
            #. movedBoxCoordinates (numpy.ndarray): The moved atoms new coordinates.
        """
        # change coordinates temporarily
        boxData = np.array(self.engine.boxCoordinates[relativeIndexes], dtype=FLOAT_TYPE)
        self.engine.boxCoordinates[relativeIndexes] = movedBoxCoordinates
        # compute after move data
        afterMoveData = np.zeros(self.__coordNumData.shape, dtype=self.__coordNumData.dtype)
        multi_atoms_coord_number_coords( indexes        = relativeIndexes,
                                         boxCoords      = self.engine.boxCoordinates,
                                         basis          = self.engine.basisVectors,
                                         isPBC          = self.engine.isPBC,
                                         coresIndexes   = self.__coresIndexes,
                                         shellsIndexes  = self.__shellsIndexes,
                                         lowerShells    = self.__lowerShells,
                                         upperShells    = self.__upperShells,
                                         asCoreDefIdxs  = self.__asCoreDefIdxs,
                                         inShellDefIdxs = self.__inShellDefIdxs,
                                         coordNumData   = afterMoveData,
                                         ncores         = self.engine._runtime_ncores)
        # reset coordinates
        self.engine.boxCoordinates[relativeIndexes] = boxData
        # set active atoms data after move
        self.set_active_atoms_data_after_move( afterMoveData )
        # compute after move standard error
        self.__coordNumDataAfterMove = self.__coordNumData-self.activeAtomsDataBeforeMove+self.activeAtomsDataAfterMove
        self.set_after_move_standard_error( self.compute_standard_error(data = self.__coordNumDataAfterMove) )

    def accept_move(self, realIndexes, relativeIndexes):
        """
        Accept move.

        :Parameters:
            #. realIndexes (numpy.ndarray): not used here.
            #. relativeIndexes (numpy.ndarray): Not used here.
        """
        self.__coordNumData = self.__coordNumDataAfterMove
        self.set_data( self.__coordNumData ) # ADDED LATER 2016-11-27 to be verified.
        # reset activeAtoms data
        self.set_active_atoms_data_before_move(None)
        self.set_active_atoms_data_after_move(None)
        # update standardError
        self.set_standard_error(self.afterMoveStandardError)
        self.set_after_move_standard_error( None )

    def reject_move(self, realIndexes, relativeIndexes):
        """
        Reject move.

        :Parameters:
            #. realIndexes (numpy.ndarray): not used here.
            #. relativeIndexes (numpy.ndarray): Not used here.
        """
        # reset activeAtoms data
        self.set_active_atoms_data_before_move(None)
        self.set_active_atoms_data_after_move(None)
        # update standardError
        self.set_after_move_standard_error( None )

    def compute_as_if_amputated(self, realIndex, relativeIndex):
        """
        Compute and return constraint's data and standard error as if atom given its
        its was amputated.

        :Parameters:
            #. realIndex (numpy.ndarray): Not used here.
            #. relativeIndex (numpy.ndarray): Not used here.
        """
        pass

    def accept_amputation(self, realIndex, relativeIndex):
        """
        Accept amputation of atom and sets constraints data and standard error accordingly.

        :Parameters:
            #. realIndex (numpy.ndarray): Atom's index as a numpy array
               of a single element.
            #. relativeIndex (numpy.ndarray): Atom's relative index as a numpy
               array of a single element.
        """
        # MAYBE WE DON"T NEED TO CHANGE DATA AND SE. BECAUSE THIS MIGHT BE A PROBLEM
        # WHEN IMPLEMENTING ATOMS RELEASING. MAYBE WE NEED TO COLLECT DATA INSTEAD, REMOVE
        # AND ADD UPON RELEASE
        self.compute_before_move(realIndexes=realIndex, relativeIndexes=relativeIndex )
        #self.compute_before_move(indexes = np.array([index], dtype=INT_TYPE) )
        # change permanently data attribute
        self.__coordNumData = self.__coordNumData-self.activeAtomsDataBeforeMove
        self.set_data( self.__coordNumData )
        self.set_active_atoms_data_before_move(None)
        self.set_standard_error( self.compute_standard_error(data = self.__coordNumData) )

    def reject_amputation(self, realIndex, relativeIndex):
        """
        Reject amputation of atom.

        :Parameters:
            #. realIndex (numpy.ndarray): Not used here.
            #. relativeIndex (numpy.ndarray): Not used here
        """
        pass

    def _on_collector_collect_atom(self, realIndex):
        # get relative index
        relativeIndex = self._atomsCollector.get_relative_index(realIndex)
        # create data dict
        dataDict = {}
        # cores indexes
        coresIndexes = []
        for idx, ci in enumerate(self.__coresIndexes):
            coresIndexes.append( np.where(ci==relativeIndex)[0] )
            ci = np.delete(ci, coresIndexes[-1], axis=0)
            ci[np.where(ci>relativeIndex)[0]] -= 1
            self.__coresIndexes[idx] = ci
        dataDict['coresIndexes'] = coresIndexes
        # shells indexes
        shellsIndexes = []
        for idx, si in enumerate(self.__shellsIndexes):
            shellsIndexes.append( np.where(si==relativeIndex)[0] )
            si = np.delete(si, shellsIndexes[-1], axis=0)
            si[np.where(si>relativeIndex)[0]] -= 1
            self.__shellsIndexes[idx] = si
        dataDict['shellsIndexes'] = shellsIndexes
        # asCorDefIdxs and inShellDefIdxs
        dataDict['asCoreDefIdxs']  = self.__asCoreDefIdxs.pop(relativeIndex)
        dataDict['inShellDefIdxs'] = self.__inShellDefIdxs.pop(relativeIndex)
        # correct number of cores without collecting
        for idx, ci in enumerate(coresIndexes):
            self.__numberOfCores[idx] -= len(ci)
        # collect atom
        self._atomsCollector.collect(realIndex, dataDict=dataDict)

    def _on_collector_release_atom(self, realIndex):
        pass


    def plot(self, ax=None, width=0.6,
                   barColor = '#99ccff',
                   cnColor  = '#ffcc00',
                   cnPtSize   = 20,
                   stdErrors = True,
                   xlabel=True, xlabelSize=16,
                   ylabel=True, ylabelSize=16,
                   legend=True, legendCols=1, legendLoc='best',
                   title=True, titleStdErr=True, titleAtRem=True,
                   titleUsedFrame=True, show=True):
        """
        Plot atomic coordination number constraint.

        :Parameters:
            #. ax (None, matplotlib Axes): matplotlib Axes instance to plot in.
               If ax is given, the figure won't be rendered and drawn.
               If None is given a new plot figure will be created and the figue
               will be rendered and drawn.
            #. width (number): Bars width, must be >0 and <=1
            #. barColor (color): boundaries bar color.
            #. cnColor (color): coordination number data points color.
            #. cnPtSize (number): coordination number data points size.
            #. stdErrors (boolean): Whether to show bars standard error.
            #. xlabel (boolean): Whether to create x label.
            #. xlabelSize (number): The x label font size.
            #. ylabel (boolean): Whether to create y label.
            #. ylabelSize (number): The y label font size.
            #. legend (boolean): Whether to create the legend or not
            #. legendCols (integer): Legend number of columns.
            #. legendLoc (string): The legend location. Anything among
               'right', 'center left', 'upper right', 'lower right', 'best',
               'center', 'lower left', 'center right', 'upper left',
               'upper center', 'lower center' is accepted.
            #. title (boolean): Whether to create the title or not
            #. titleStdErr (boolean): Whether to show constraint standard error
               value in title.
            #. titleAtRem (boolean): Whether to show engine's number of removed
               atoms.
            #. titleUsedFrame(boolean): Whether to show used frame name in
               title.
            #. show (boolean): Whether to render and show figure before
               returning.

        :Returns:
            #. figure (matplotlib Figure): matplotlib used figure.
            #. axes (matplotlib Axes): matplotlib used axes.
        """
        # get constraint value
        if all([d is None for d in self.data]):
            LOGGER.warn("%s constraint data are not computed."%(self.__class__.__name__))
            return
        # check width
        assert 0<width<=1, LOGGER.error("width must be a number between 0 and 1")
        # import matplotlib
        import matplotlib.pyplot as plt
        # get axes
        if ax is None:
            FIG  = plt.figure()
            AXES = plt.gca()
        else:
            AXES = ax
            FIG  = AXES.get_figure()
        # plot bars
        ind    = np.arange(1,len(self.data)+1)  # the x locations for the groups
        bottom = self.minAtoms
        height = [self.maxAtoms[idx]-self.minAtoms[idx] for idx in xrange(len(self.maxAtoms))]
        p = AXES.bar(ind+width/2., height, width, bottom=self.minAtoms, color=barColor, label="boundaries")
        # add coordination number points
        CN = self.data/self.__numberOfCores
        AXES.plot(ind+width/2., CN, 'o', label="mean coord num", color=cnColor, markersize=cnPtSize, markevery=1 )
        # set ticks
        plt.xticks(ind+width/2., ["%s-%s"%(e[:2]) for e in self.__coordNumDef])
        # compute standard errors
        if stdErrors:
            StdErrs  = []
            for idx, cn in enumerate( CN ):
                if cn < self.__minAtoms[idx]:
                    StdErrs.append( self.__weights[idx]*(self.__minAtoms[idx]-cn) )
                elif cn > self.__maxAtoms[idx]:
                    StdErrs.append( self.__weights[idx]*(cn-self.__maxAtoms[idx]) )
                else:
                    StdErrs.append( 0. )
            for mi,ma, std, rect in zip(self.__minAtoms,self.__maxAtoms,StdErrs, AXES.patches):
                height = rect.get_height()
                t = AXES.text(x     = rect.get_x() + rect.get_width()/2,
                              y     = float(ma+mi)/2.,
                              s     = " "+str(std),
                              color = 'black',
                              rotation = 90,
                              horizontalalignment = 'center',
                              verticalalignment   = 'center')
        # set limits
        minY = min([min(CN),min(self.minAtoms)])
        maxY = max([max(CN),max(self.maxAtoms)])
        AXES.set_xlim(0,len(self.data)+1.5)
        AXES.set_ylim(minY-1,maxY+1)
        # set axis labels
        if xlabel:
            AXES.set_xlabel("Core-Shell atoms", size=xlabelSize)
        if ylabel:
            AXES.set_ylabel("Coordination number"  , size=ylabelSize)
        # set title
        if title:
            FIG.canvas.set_window_title('Atomic Coordination Number Constraint')
            if titleUsedFrame:
                t = '$frame: %s$ : '%self.engine.usedFrame.replace('_','\\_')
            else:
                t = ''
            if titleAtRem:
                t += "$%i$ $rem.$ $at.$ - "%(len(self.engine._atomsCollector))
            if titleStdErr and self.standardError is not None:
                t += "$std$ $error=%.6f$ "%(self.standardError)
            if len(t):
                AXES.set_title(t)
        # set background color
        FIG.patch.set_facecolor('white')
        # plot legend
        if legend:
            AXES.legend(frameon=False, ncol=legendCols, loc=legendLoc, numpoints=1)
        #show
        if show:
            plt.show()
        # return axes
        return FIG, AXES

    def export(self, fname, delimiter='     ', comments='# '):
        """
        Export atomic coordination number constraint data.

        :Parameters:
            #. fname (path): full file name and path.
            #. delimiter (string): String or character separating columns.
            #. comments (string): String that will be prepended to the header.
        """
        # get constraint value
        if all([d is None for d in self.data]):
            LOGGER.warn("%s constraint data are not computed."%(self.__class__.__name__))
            return
        CN = self.data/self.__numberOfCores
        StdErrs  = []
        for idx, cn in enumerate( CN ):
            if cn < self.__minAtoms[idx]:
                StdErrs.append( self.__weights[idx]*(self.__minAtoms[idx]-cn) )
            elif cn > self.__maxAtoms[idx]:
                StdErrs.append( self.__weights[idx]*(cn-self.__maxAtoms[idx]) )
            else:
                StdErrs.append( 0. )
        # create header
        header = ["core-shell","ninimum_coord_num","naximum_coord_num","mean_coord_num","standard_error"]
        # create data lists
        data = [["%s-%s"%(e[:2]) for e in self.__coordNumDef],
                 [str(i) for i in self.__minAtoms],
                 [str(i) for i in self.__maxAtoms],
                 [str(i) for i in CN],
                 [str(i) for i in StdErrs]]
        # save
        data = np.transpose(data)
        np.savetxt(fname     = fname,
                   X         = data,
                   fmt       = '%s',
                   delimiter = delimiter,
                   header    = " ".join(header),
                   comments  = comments)



"""
ImproperAngleConstraints contains classes for all constraints related to
improper angles between atoms.

.. inheritance-diagram:: fullrmc.Constraints.ImproperAngleConstraints
    :parts: 1
"""
# standard libraries imports
from __future__ import print_function

# external libraries imports
import numpy as np

# fullrmc imports
from ..Globals import INT_TYPE, FLOAT_TYPE, PI, LOGGER
from ..Globals import str, long, unicode, bytes, basestring, range, xrange, maxint
from ..Core.Collection import is_number, is_integer, raise_if_collected, reset_if_collected_out_of_date
from ..Core.Collection import get_caller_frames
from ..Core.Constraint import Constraint, SingularConstraint, RigidConstraint
from ..Core.improper_angles import full_improper_angles_coords




class ImproperAngleConstraint(RigidConstraint, SingularConstraint):
    """
    Controls the improper angle formed with 4 defined atoms. It's mainly used
    to keep the improper atom in the plane defined with three other atoms.
    The improper vector is defined as the vector from the first atom of the
    plane to the improper atom. Therefore the improper angle is defined between
    the improper vector and the plane.


    +--------------------------------------------------------------------------+
    |.. figure:: improperSketch.png                                            |
    |   :width: 269px                                                          |
    |   :height: 200px                                                         |
    |   :align: center                                                         |
    |                                                                          |
    |   Improper angle sketch defined between four atoms.                      |
    +--------------------------------------------------------------------------+


     .. raw:: html

        <iframe width="560" height="315"
        src="https://www.youtube.com/embed/qVATE-9cIBg"
        frameborder="0" allowfullscreen>
        </iframe>

    :Parameters:
        #. rejectProbability (Number): rejecting probability of all steps
           where standardError increases. It must be between 0 and 1 where 1
           means rejecting all steps where standardError increases and 0 means
           accepting all steps regardless whether standardError increases or
           not.

    .. code-block:: python

        ## Tetrahydrofuran (THF) molecule sketch
        ##
        ##              O
        ##   H41      /   \      H11
        ##      \  /         \  /
        ## H42-- C4    THF    C1 --H12
        ##        \  MOLECULE /
        ##         \         /
        ##   H31-- C3-------C2 --H21
        ##        /         \\
        ##     H32            H22
        ##

        # import fullrmc modules
        from fullrmc.Engine import Engine
        from fullrmc.Constraints.ImproperAngleConstraints import ImproperAngleConstraint

        # create engine
        ENGINE = Engine(path='my_engine.rmc')

        # set pdb file
        ENGINE.set_pdb('system.pdb')

        # create and add constraint
        IAC = ImproperAngleConstraint()
        ENGINE.add_constraints(IAC)

        # define intra-molecular improper angles
        IAC.create_angles_by_definition( anglesDefinition={"THF": [ ('C2','O','C1','C4', -15, 15),
                                                                    ('C3','O','C1','C4', -15, 15) ] })


    """

    def __init__(self, rejectProbability=1):
        # initialize constraint
        RigidConstraint.__init__(self, rejectProbability=rejectProbability)
        # set atomsCollector data keys
        self._atomsCollector.set_data_keys( ['improperMap','otherMap'] )
        # init angles data
        self.__anglesList = [[],[],[],[],[],[]]
        self.__angles     = {}
        # set computation cost
        self.set_computation_cost(3.0)
        # create dump flag
        self.__dumpAngles = True
        # set frame data
        FRAME_DATA = [d for d in self.FRAME_DATA]
        FRAME_DATA.extend(['_ImproperAngleConstraint__anglesList',
                           '_ImproperAngleConstraint__angles',] )
        RUNTIME_DATA = [d for d in self.RUNTIME_DATA]
        RUNTIME_DATA.extend( [] )
        object.__setattr__(self, 'FRAME_DATA',   tuple(FRAME_DATA) )
        object.__setattr__(self, 'RUNTIME_DATA', tuple(RUNTIME_DATA) )

    @property
    def anglesList(self):
        """ Get improper angles list."""
        return self.__anglesList

    @property
    def angles(self):
        """ Get angles dictionary for every and each atom."""
        return self.__angles

    def _on_collector_reset(self):
        self._atomsCollector._randomData = set([])

    def listen(self, message, argument=None):
        """
        Listens to any message sent from the Broadcaster.

        :Parameters:
            #. message (object): Any python object to send to constraint's
               listen method.
            #. argument (object): Any type of argument to pass to the
               listeners.
        """
        if message in("engine set","update boundary conditions",):
            # set angles and reset constraint
            AL = [ self.__anglesList[0],self.__anglesList[1],
                   self.__anglesList[2],self.__anglesList[3],
                   [a*FLOAT_TYPE(180.)/PI for a in self.__anglesList[4]],
                   [a*FLOAT_TYPE(180.)/PI for a in self.__anglesList[5]] ]
            self.set_angles(anglesList=AL, tform=False)
            # reset constraint is called in set_angles

    #@raise_if_collected
    def set_angles(self, anglesList, tform=True):
        """
        Set angles dictionary by parsing anglesList list.

        :Parameters:
            #. anglesList (list): Angles list definition.

               tuples format: every item must be a list of five items.\n
               #. Improper atom index that must be in the plane.
               #. Index of atom 'O' considered the origin of the plane.
               #. Index of atom 'x' used to calculated 'Ox' vector.
               #. Index of atom 'y' used to calculated 'Oy' vector.
               #. Minimum lower limit or minimum angle allowed
                  in degrees which later will be converted to rad.
               #. Maximum upper limit or maximum angle allowed
                  in degrees which later will be converted to rad.

               six vectors format: every item must be a list of five items.\n
               #. List containing improper atoms index that must be in the
                  plane.
               #. List containing index of atoms 'O' considered the origin
                  of the plane.
               #. List containing index of atoms 'x' used to calculated
                  'Ox' vector.
               #. List containing index of atom 'y' used to calculated 'Oy'
                  vector.
               #. List containing minimum lower limit or minimum angle allowed
                  in degrees which later will be converted to rad.
               #. List containing maximum upper limit or maximum angle allowed
                  in degrees which later will be converted to rad.

           #. tform (boolean): Whether given anglesList follows tuples format,
              If not then it must follow the six vectors one.
        """
        # check if bondsList is given
        if anglesList is None:
            anglesList = [[],[],[],[],[],[]]
            tform      = False
        elif len(anglesList) == 6 and len(anglesList[0]) == 0:
            tform     = False
        if self.engine is None:
            self.__anglesList = anglesList
            self.__angles     = {}
        else:
            NUMBER_OF_ATOMS   = self.engine.get_original_data("numberOfAtoms")
            oldAnglesList = self.__anglesList
            oldAngles     = self.__angles
            self.__anglesList = [np.array([], dtype=INT_TYPE),
                                 np.array([], dtype=INT_TYPE),
                                 np.array([], dtype=INT_TYPE),
                                 np.array([], dtype=INT_TYPE),
                                 np.array([], dtype=FLOAT_TYPE),
                                 np.array([], dtype=FLOAT_TYPE)]
            self.__angles     = {}
            self.__dumpAngles = False
            # build angles
            try:
                if tform:
                    for angle in anglesList:
                        self.add_angle(angle)
                else:
                    for idx in xrange(len(anglesList[0])):
                        angle = [anglesList[0][idx], anglesList[1][idx], anglesList[2][idx], anglesList[3][idx], anglesList[4][idx], anglesList[5][idx]]
                        self.add_angle(angle)
            except Exception as e:
                self.__dumpAngles = True
                self.__anglesList = oldAnglesList
                self.__angles     = oldAngles
                LOGGER.error(e)
                import traceback
                raise Exception(traceback.format_exc())
            self.__dumpAngles = True
            # finalize angles
            for idx in xrange(NUMBER_OF_ATOMS):
                self.__angles[INT_TYPE(idx)] = self.__angles.get(INT_TYPE(idx), {"oIdx":[],"xIdx":[],"yIdx":[],"improperMap":[],"otherMap":[]}  )
        # dump to repository
        self._dump_to_repository({'_ImproperAngleConstraint__anglesList' :self.__anglesList,
                                  '_ImproperAngleConstraint__angles'     :self.__angles})
        # reset constraint
        self.reset_constraint()

    #@raise_if_collected
    def add_angle(self, angle):
        """
        Add a single angle to the list of constraint angles. All angles are in degrees.

        :Parameters:
            #. angle (list): The angle list of five items.\n
               #. Improper atom index that must be in the plane.
               #. Index of atom 'O' considered the origin of the plane.
               #. Index of atom 'x' used to calculated 'Ox' vector.
               #. Index of atom 'y' used to calculated 'Oy' vector.
               #. Minimum lower limit or minimum angle allowed
                  in degrees which later will be converted to rad.
               #. Maximum upper limit or maximum angle allowed
                  in degrees which later will be converted to rad.
        """
        assert self.engine is not None, LOGGER.error("setting an angle is not allowed unless engine is defined.")
        NUMBER_OF_ATOMS = self.engine.get_original_data("numberOfAtoms")
        assert isinstance(angle, (list, set, tuple)), LOGGER.error("anglesList items must be lists")
        assert len(angle)==6, LOGGER.error("anglesList items must be lists of 6 items each")
        improperIdx, oIdx, xIdx, yIdx, lower, upper = angle
        assert is_integer(improperIdx), LOGGER.error("angle first item must be an integer")
        improperIdx = INT_TYPE(improperIdx)
        assert is_integer(oIdx), LOGGER.error("angle second item must be an integer")
        oIdx = INT_TYPE(oIdx)
        assert is_integer(xIdx), LOGGER.error("angle third item must be an integer")
        xIdx = INT_TYPE(xIdx)
        assert is_integer(yIdx), LOGGER.error("angle fourth item must be an integer")
        yIdx = INT_TYPE(yIdx)
        assert improperIdx>=0, LOGGER.error("angle first item must be positive")
        assert improperIdx<NUMBER_OF_ATOMS, LOGGER.error("angle first item atom index must be smaller than maximum number of atoms")
        assert oIdx>=0, LOGGER.error("angle second item must be positive")
        assert oIdx<NUMBER_OF_ATOMS, LOGGER.error("angle second item atom index must be smaller than maximum number of atoms")
        assert xIdx>=0, LOGGER.error("angle third item must be positive")
        assert xIdx<NUMBER_OF_ATOMS, LOGGER.error("angle third item atom index must be smaller than maximum number of atoms")
        assert yIdx>=0, LOGGER.error("angle fourth item must be positive")
        assert yIdx<NUMBER_OF_ATOMS, LOGGER.error("angle atom index must be smaller than maximum number of atoms")
        assert improperIdx!=oIdx, LOGGER.error("angle second items can't be the same")
        assert improperIdx!=xIdx, LOGGER.error("angle third items can't be the same")
        assert improperIdx!=yIdx, LOGGER.error("angle fourth items can't be the same")
        assert oIdx!=xIdx, LOGGER.error("angle second and third items can't be the same")
        assert oIdx!=yIdx, LOGGER.error("angle second and fourth items can't be the same")
        assert xIdx!=yIdx, LOGGER.error("angle third and fourth items can't be the same")
        assert is_number(lower), LOGGER.error("angle fifth item must be a number")
        lower = FLOAT_TYPE(lower)
        assert is_number(upper), LOGGER.error("angle sixth item must be a number")
        upper = FLOAT_TYPE(upper)
        assert lower>=-90, LOGGER.error("angle fifth item must be bigger or equal to -90 deg.")
        assert upper>lower, LOGGER.error("angle fifth item must be smaller than the sixth item")
        assert upper<=90, LOGGER.error("angle sixth item must be smaller or equal to 90")
        lower *= FLOAT_TYPE( PI/FLOAT_TYPE(180.) )
        upper *= FLOAT_TYPE( PI/FLOAT_TYPE(180.) )
        # create improper angle
        if not improperIdx in self.__angles:
            anglesImproper = {"oIdx":[],"xIdx":[],"yIdx":[],"improperMap":[],"otherMap":[]}
        else:
            anglesImproper = {"oIdx"        :self.__angles[improperIdx]["oIdx"],
                              "xIdx"        :self.__angles[improperIdx]["xIdx"],
                              "yIdx"        :self.__angles[improperIdx]["yIdx"],
                              "improperMap" :self.__angles[improperIdx]["improperMap"],
                              "otherMap"    :self.__angles[improperIdx]["otherMap"] }
        # create anglesO angle
        if not oIdx in self.__angles:
            anglesO = {"oIdx":[],"xIdx":[],"yIdx":[],"improperMap":[],"otherMap":[]}
        else:
            anglesO = {"oIdx"        :self.__angles[oIdx]["oIdx"],
                       "xIdx"        :self.__angles[oIdx]["xIdx"],
                       "yIdx"        :self.__angles[oIdx]["yIdx"],
                       "improperMap" :self.__angles[oIdx]["improperMap"],
                       "otherMap"    :self.__angles[oIdx]["otherMap"] }
        # create anglesX angle
        if not xIdx in self.__angles:
            anglesX = {"oIdx":[],"xIdx":[],"yIdx":[],"improperMap":[],"otherMap":[]}
        else:
            anglesX = {"oIdx"        :self.__angles[xIdx]["oIdx"],
                       "xIdx"        :self.__angles[xIdx]["xIdx"],
                       "yIdx"        :self.__angles[xIdx]["yIdx"],
                       "improperMap" :self.__angles[xIdx]["improperMap"],
                       "otherMap"    :self.__angles[xIdx]["otherMap"] }
        # create anglesY angle
        if not yIdx in self.__angles:
            anglesY = {"oIdx":[],"xIdx":[],"yIdx":[],"improperMap":[],"otherMap":[]}
        else:
            anglesY = {"oIdx"        :self.__angles[yIdx]["oIdx"],
                       "xIdx"        :self.__angles[yIdx]["xIdx"],
                       "yIdx"        :self.__angles[yIdx]["yIdx"],
                       "improperMap" :self.__angles[yIdx]["improperMap"],
                       "otherMap"    :self.__angles[yIdx]["otherMap"] }
        # check for re-defining
        setPos = oPos = xPos = yPos = None
        if oIdx in anglesImproper["oIdx"] and xIdx in anglesImproper["xIdx"] and yIdx in anglesImproper["yIdx"]:
            oPos = anglesImproper["oIdx"].index(oIdx)
            xPos = anglesImproper["xIdx"].index(xIdx)
            yPos = anglesImproper["yIdx"].index(yIdx)
        elif oIdx in anglesImproper["oIdx"] and yIdx in anglesImproper["xIdx"] and xIdx in anglesImproper["yIdx"]:
            oPos = anglesImproper["oIdx"].index(oIdx)
            xPos = anglesImproper["yIdx"].index(xIdx)
            yPos = anglesImproper["xIdx"].index(yIdx)
        elif xIdx in anglesImproper["oIdx"] and oIdx in anglesImproper["xIdx"] and yIdx in anglesImproper["yIdx"]:
            oPos = anglesImproper["xIdx"].index(oIdx)
            xPos = anglesImproper["oIdx"].index(xIdx)
            yPos = anglesImproper["yIdx"].index(yIdx)
        elif yIdx in anglesImproper["oIdx"] and xIdx in anglesImproper["xIdx"] and oIdx in anglesImproper["yIdx"]:
            oPos = anglesImproper["yIdx"].index(oIdx)
            xPos = anglesImproper["xIdx"].index(xIdx)
            yPos = anglesImproper["oIdx"].index(yIdx)
        if oPos is not None and (oPos==xPos) and (oPos==yPos):
            LOGGER.warn("Angle definition for improper atom index '%i' and O '%i' and X '%i' and Y '%i' atoms is  already defined. New angle limits [%.3f,%.3f] are set."%(improperIdx, oIdx, xIdx, yIdx, lower, upper))
            setPos = anglesImproper["improperMap"][oPos]
        # set angle
        if setPos is None:
            anglesImproper["oIdx"].append(oIdx)
            anglesImproper["xIdx"].append(xIdx)
            anglesImproper["yIdx"].append(yIdx)
            anglesImproper["improperMap"].append( len(self.__anglesList[0]) )
            anglesO["otherMap"].append( len(self.__anglesList[0]) )
            anglesX["otherMap"].append( len(self.__anglesList[0]) )
            anglesY["otherMap"].append( len(self.__anglesList[0]) )
            self.__anglesList[0] = np.append(self.__anglesList[0],improperIdx)
            self.__anglesList[1] = np.append(self.__anglesList[1],oIdx)
            self.__anglesList[2] = np.append(self.__anglesList[2],xIdx)
            self.__anglesList[3] = np.append(self.__anglesList[3],yIdx)
            self.__anglesList[4] = np.append(self.__anglesList[4],lower)
            self.__anglesList[5] = np.append(self.__anglesList[5],upper)

        else:
            assert self.__anglesList[0][setPos] == improperIdx, LOGGER.error("mismatched angles improper atom '%s' and '%s'"%(self.__anglesList[0][setPos],improperIdx))
            assert sorted([oIdx, xIdx, yIdx]) == sorted([self.__anglesList[1][setPos],self.__anglesList[2][setPos],self.__anglesList[3][setPos]]), LOGGER.error("mismatched angles O, Y and Y at improper atom '%s'"%(improperIdx))
            self.__anglesList[1][setPos] = oIdx
            self.__anglesList[2][setPos] = xIdx
            self.__anglesList[3][setPos] = yIdx
            self.__anglesList[4][setPos] = lower
            self.__anglesList[5][setPos] = upper
        self.__angles[improperIdx] = anglesImproper
        self.__angles[oIdx]        = anglesO
        self.__angles[xIdx]        = anglesX
        self.__angles[yIdx]        = anglesY
        # dump to repository
        if self.__dumpAngles:
            self._dump_to_repository({'_ImproperAngleConstraint__anglesList' :self.__anglesList,
                                      '_ImproperAngleConstraint__angles'     :self.__angles})
            # reset constraint
            self.reset_constraint()

    #@raise_if_collected
    def create_angles_by_definition(self, anglesDefinition):
        """
        Creates anglesList using angles definition. This calls set_angles(anglesMap)
        and generates angles attribute. All angles are in degrees.

        :Parameters:
            #. anglesDefinition (dict): Angles definition.
               Every key must be a molecule name.
               Every key value must be a list of angles definitions.
               Every angle definition is a list of five items where:

               #. Name of improper atom that must be in the plane.
               #. Name of atom 'O' considered the origin of the plane.
               #. Name of atom 'x' used to calculated 'Ox' vector.
               #. Name of atom 'y' used to calculated 'Oy' vector.
               #. Minimum lower limit or minimum angle allowed
                  in degrees which later will be converted to rad.
               #. Maximum upper limit or maximum angle allowed
                  in degrees which later will be converted to rad.

        ::

            e.g. (Benzene):  anglesDefinition={"BENZ": [('C3','C1','C2','C6', -10, 10),
                                                        ('C4','C1','C2','C6', -10, 10),
                                                        ('C5','C1','C2','C6', -10, 10) ] }

        """
        if self.engine is None:
            raise Exception("Engine is not defined. Can't create impoper angles by definition")
        assert isinstance(anglesDefinition, dict), "anglesDefinition must be a dictionary"
        # check map definition
        ALL_NAMES       = self.engine.get_original_data("allNames")
        NUMBER_OF_ATOMS = self.engine.get_original_data("numberOfAtoms")
        MOLECULES_NAME  = self.engine.get_original_data("moleculesName")
        MOLECULES_INDEX = self.engine.get_original_data("moleculesIndex")
        existingMoleculesName = sorted(set(MOLECULES_NAME))
        anglesDef = {}
        for mol in anglesDefinition:
            angles = anglesDefinition[mol]
            if mol not in existingMoleculesName:
                log.LocalLogger("fullrmc").logger.warn("Molecule name '%s' in anglesDefinition is not recognized, angles definition for this particular molecule is omitted"%str(mol))
                continue
            assert isinstance(angles, (list, set, tuple)), LOGGER.error("mapDefinition molecule angles must be a list")
            angles = list(angles)
            molAnglesList = []
            for angle in angles:
                assert isinstance(angle, (list, set, tuple)), LOGGER.error("mapDefinition angles must be a list")
                angle = list(angle)
                assert len(angle)==6
                improperAt, oAt, xAt, yAt, lower, upper = angle
                # check for redundancy
                append = True
                for b in molAnglesList:
                    if (b[0]==improperAt):
                        if sorted([oAt,xAt,yAt]) == sorted([b[1],b[2],b[3]]):
                            LOGGER.warn("Redundant definition for anglesDefinition found. The later '%s' is ignored"%str(b))
                            append = False
                            break
                if append:
                    molAnglesList.append((improperAt, oAt, xAt, yAt, lower, upper))
            # create bondDef for molecule mol
            anglesDef[mol] = molAnglesList
        # create mols dictionary
        mols = {}
        for idx in xrange(NUMBER_OF_ATOMS):
            molName = MOLECULES_NAME[idx]
            if not molName in anglesDef:
                continue
            molIdx = MOLECULES_INDEX[idx]
            if not molIdx in mols:
                mols[molIdx] = {"name":molName, "indexes":[], "names":[]}
            mols[molIdx]["indexes"].append(idx)
            mols[molIdx]["names"].append(ALL_NAMES[idx])
        # get anglesList
        anglesList = []
        for val in mols.values():
            indexes = val["indexes"]
            names   = val["names"]
            # get definition for this molecule
            thisDef = anglesDef[val["name"]]
            for angle in thisDef:
                improperIdx = indexes[ names.index(angle[0]) ]
                oIdx        = indexes[ names.index(angle[1]) ]
                xIdx        = indexes[ names.index(angle[2]) ]
                yIdx        = indexes[ names.index(angle[3]) ]
                lower       = angle[4]
                upper       = angle[5]
                anglesList.append((improperIdx, oIdx, xIdx, yIdx, lower, upper))
        # create angles
        self.set_angles(anglesList=anglesList)

    def compute_standard_error(self, data):
        """
        Compute the standard error (StdErr) of data not satisfying constraint's
        conditions.

        .. math::
            StdErr = \\sum \\limits_{i}^{C}
            ( \\theta_{i} - \\theta_{i}^{min} ) ^{2}
            \\int_{0}^{\\theta_{i}^{min}} \\delta(\\theta-\\theta_{i}) d \\theta
            +
            ( \\theta_{i} - \\theta_{i}^{max} ) ^{2}
            \\int_{\\theta_{i}^{max}}^{\\pi} \\delta(\\theta-\\theta_{i}) d \\theta

        Where:\n
        :math:`C` is the total number of defined improper angles constraints. \n
        :math:`\\theta_{i}^{min}` is the improper angle constraint lower limit set for constraint i. \n
        :math:`\\theta_{i}^{max}` is the improper angle constraint upper limit set for constraint i. \n
        :math:`\\theta_{i}` is the improper angle computed for constraint i. \n
        :math:`\\delta` is the Dirac delta function. \n
        :math:`\\int_{0}^{\\theta_{i}^{min}} \\delta(\\theta-\\theta_{i}) d \\theta`
        is equal to 1 if :math:`0 \\leqslant \\theta_{i} \\leqslant \\theta_{i}^{min}` and 0 elsewhere.\n
        :math:`\\int_{\\theta_{i}^{max}}^{\\pi} \\delta(\\theta-\\theta_{i}) d \\theta`
        is equal to 1 if :math:`\\theta_{i}^{max} \\leqslant \\theta_{i} \\leqslant \\pi` and 0 elsewhere.\n

        :Parameters:
            #. data (numpy.array): data to compute standardError.

        :Returns:
            #. standardError (number): computed standardError of given data.
        """
        return FLOAT_TYPE( np.sum(data["reducedAngles"]**2) )

    def get_constraint_value(self):
        """
        get constraint's data value.

        :Returns:
            #. data (dictionary): constraint data.
        """
        return self.data

    @reset_if_collected_out_of_date
    def compute_data(self):
        """ Compute constraint's data."""
        if len(self._atomsCollector):
            anglesData    = np.zeros(self.__anglesList[0].shape[0], dtype=FLOAT_TYPE)
            reducedData   = np.zeros(self.__anglesList[0].shape[0], dtype=FLOAT_TYPE)
            anglesIndexes = set(set(range(self.__anglesList[0].shape[0])))
            anglesIndexes  = list( anglesIndexes-self._atomsCollector._randomData )
            improperIdxs = self._atomsCollector.get_relative_indexes(self.__anglesList[0][anglesIndexes])
            oIdxs        = self._atomsCollector.get_relative_indexes(self.__anglesList[1][anglesIndexes])
            xIdxs        = self._atomsCollector.get_relative_indexes(self.__anglesList[2][anglesIndexes])
            yIdxs        = self._atomsCollector.get_relative_indexes(self.__anglesList[3][anglesIndexes])
            lowerLimit = self.__anglesList[4][anglesIndexes]
            upperLimit = self.__anglesList[5][anglesIndexes]
        else:
            improperIdxs = self._atomsCollector.get_relative_indexes(self.__anglesList[0])
            oIdxs        = self._atomsCollector.get_relative_indexes(self.__anglesList[1])
            xIdxs        = self._atomsCollector.get_relative_indexes(self.__anglesList[2])
            yIdxs        = self._atomsCollector.get_relative_indexes(self.__anglesList[3])
            lowerLimit = self.__anglesList[4]
            upperLimit = self.__anglesList[5]
        # compute data
        angles, reduced =  full_improper_angles_coords( improperIdxs       = improperIdxs,
                                                        oIdxs              = oIdxs,
                                                        xIdxs              = xIdxs,
                                                        yIdxs              = yIdxs,
                                                        lowerLimit         = lowerLimit,
                                                        upperLimit         = upperLimit,
                                                        boxCoords          = self.engine.boxCoordinates,
                                                        basis              = self.engine.basisVectors,
                                                        isPBC              = self.engine.isPBC,
                                                        reduceAngleToUpper = False,
                                                        reduceAngleToLower = False,
                                                        ncores             = INT_TYPE(1))
        # create full length data
        if len(self._atomsCollector):
            anglesData[anglesIndexes]  = angles
            reducedData[anglesIndexes] = reduced
            angles  = anglesData
            reduced = reducedData
        # set data.
        self.set_data( {"angles":angles, "reducedAngles":reduced} )
        self.set_active_atoms_data_before_move(None)
        self.set_active_atoms_data_after_move(None)
        # set standardError
        self.set_standard_error( self.compute_standard_error(data = self.data) )
        # set original data
        if self.originalData is None:
            self._set_original_data(self.data)

    def compute_before_move(self, realIndexes, relativeIndexes):
        """
        Compute constraint's data before move is executed.

        :Parameters:
            #. realIndexes (numpy.ndarray): Group atoms index the move will
               be applied to.
            #. relativeIndexes (numpy.ndarray): Not used here.
        """
        # get angles indexes
        anglesIndexes = []
        #for idx in relativeIndexes:
        for idx in realIndexes:
            anglesIndexes.extend( self.__angles[idx]['improperMap'] )
            anglesIndexes.extend( self.__angles[idx]['otherMap'] )
        #anglesIndexes = list(set(anglesIndexes))
        anglesIndexes = list( set(anglesIndexes)-set(self._atomsCollector._randomData) )
        # compute data before move
        if len(anglesIndexes):
            angles, reduced =  full_improper_angles_coords( improperIdxs       = self._atomsCollector.get_relative_indexes(self.__anglesList[0][anglesIndexes]),
                                                            oIdxs              = self._atomsCollector.get_relative_indexes(self.__anglesList[1][anglesIndexes]),
                                                            xIdxs              = self._atomsCollector.get_relative_indexes(self.__anglesList[2][anglesIndexes]),
                                                            yIdxs              = self._atomsCollector.get_relative_indexes(self.__anglesList[3][anglesIndexes]),
                                                            lowerLimit         = self.__anglesList[4][anglesIndexes],
                                                            upperLimit         = self.__anglesList[5][anglesIndexes],
                                                            boxCoords          = self.engine.boxCoordinates,
                                                            basis              = self.engine.basisVectors ,
                                                            isPBC              = self.engine.isPBC,
                                                            reduceAngleToUpper = False,
                                                            reduceAngleToLower = False,
                                                            ncores             = INT_TYPE(1))
        else:
            angles  = None
            reduced = None
        # set data before move
        self.set_active_atoms_data_before_move( {"anglesIndexes":anglesIndexes, "angles":angles, "reducedAngles":reduced} )
        self.set_active_atoms_data_after_move(None)

    def compute_after_move(self, realIndexes, relativeIndexes, movedBoxCoordinates):
        """
        Compute constraint's data after move is executed.

        :Parameters:
            #. realIndexes (numpy.ndarray): Not used here.
            #. relativeIndexes (numpy.ndarray): Group atoms relative index the
               move will be applied to.
            #. movedBoxCoordinates (numpy.ndarray): The moved atoms new
               coordinates.
        """
        # get angles indexes
        anglesIndexes = self.activeAtomsDataBeforeMove["anglesIndexes"]
        # change coordinates temporarily
        boxData = np.array(self.engine.boxCoordinates[relativeIndexes], dtype=FLOAT_TYPE)
        self.engine.boxCoordinates[relativeIndexes] = movedBoxCoordinates
        # compute data before move
        if len(anglesIndexes):
            angles, reduced =  full_improper_angles_coords( improperIdxs       = self._atomsCollector.get_relative_indexes(self.__anglesList[0][anglesIndexes]),
                                                            oIdxs              = self._atomsCollector.get_relative_indexes(self.__anglesList[1][anglesIndexes]),
                                                            xIdxs              = self._atomsCollector.get_relative_indexes(self.__anglesList[2][anglesIndexes]),
                                                            yIdxs              = self._atomsCollector.get_relative_indexes(self.__anglesList[3][anglesIndexes]),
                                                            lowerLimit         = self.__anglesList[4][anglesIndexes],
                                                            upperLimit         = self.__anglesList[5][anglesIndexes],
                                                            boxCoords          = self.engine.boxCoordinates,
                                                            basis              = self.engine.basisVectors ,
                                                            isPBC              = self.engine.isPBC,
                                                            reduceAngleToUpper = False,
                                                            reduceAngleToLower = False,
                                                            ncores             = INT_TYPE(1))
        else:
            angles  = None
            reduced = None
        # set active data after move
        self.set_active_atoms_data_after_move( {"angles":angles, "reducedAngles":reduced} )
        # reset coordinates
        self.engine.boxCoordinates[relativeIndexes] = boxData
        # compute standardError after move
        if angles is None:
            self.set_after_move_standard_error( self.standardError )
        else:
            # anglesIndexes is a fancy slicing, RL is a copy not a view.
            RL = self.data["reducedAngles"][anglesIndexes]
            self.data["reducedAngles"][anglesIndexes] += reduced-self.activeAtomsDataBeforeMove["reducedAngles"]
            self.set_after_move_standard_error( self.compute_standard_error(data = self.data) )
            self.data["reducedAngles"][anglesIndexes] = RL

    def accept_move(self, realIndexes, relativeIndexes):
        """
        Accept move

        :Parameters:
            #. realIndexes (numpy.ndarray): Not used here.
            #. relativeIndexes (numpy.ndarray): Not used here.
        """
        # get indexes
        anglesIndexes = self.activeAtomsDataBeforeMove["anglesIndexes"]
        if len(anglesIndexes):
            # set new data
            data = self.data
            data["angles"][anglesIndexes]        += self.activeAtomsDataAfterMove["angles"]-self.activeAtomsDataBeforeMove["angles"]
            data["reducedAngles"][anglesIndexes] += self.activeAtomsDataAfterMove["reducedAngles"]-self.activeAtomsDataBeforeMove["reducedAngles"]
            self.set_data( data )
            # update standardError
            self.set_standard_error( self.afterMoveStandardError )
        # reset activeAtoms data
        self.set_active_atoms_data_before_move(None)
        self.set_active_atoms_data_after_move(None)

    def reject_move(self, realIndexes, relativeIndexes):
        """
        Reject move

        :Parameters:
            #. realIndexes (numpy.ndarray): Not used here.
            #. relativeIndexes (numpy.ndarray): Not used here.
        """
        # reset activeAtoms data
        self.set_active_atoms_data_before_move(None)
        self.set_active_atoms_data_after_move(None)

    def accept_amputation(self, realIndex, relativeIndex):
        """
        Accept amputation of atom and sets constraint's data and standard
        error accordingly.

        :Parameters:
            #. realIndex (numpy.ndarray): Atom's index as a numpy array
               of a single element.
            #. relativeIndex (numpy.ndarray): Not used here.
        """
        # MAYBE WE DON"T NEED TO CHANGE DATA AND SE. BECAUSE THIS MIGHT BE A PROBLEM
        # WHEN IMPLEMENTING ATOMS RELEASING. MAYBE WE NEED TO COLLECT DATA INSTEAD, REMOVE
        # AND ADD UPON RELEASE
        # get all involved data
        anglesIndexes = []
        for idx in realIndex:
            anglesIndexes.extend( self.__angles[idx]['improperMap'] )
            anglesIndexes.extend( self.__angles[idx]['otherMap'] )
        anglesIndexes = list(set(anglesIndexes))
        if len(anglesIndexes):
            # set new data
            data = self.data
            data["angles"][anglesIndexes]        = 0
            data["reducedAngles"][anglesIndexes] = 0
            self.set_data( data )
            # update standardError
            SE = self.compute_standard_error(data = self.get_constraint_value())
            self.set_standard_error( SE )
        # reset activeAtoms data
        self.set_active_atoms_data_before_move(None)

    def reject_amputation(self, realIndex, relativeIndex):
        """
        Reject amputation of atom.

        :Parameters:
            #. realIndex (numpy.ndarray): Not used here.
            #. relativeIndex (numpy.ndarray): Not used here.
        """
        pass

    def _on_collector_collect_atom(self, realIndex):
        # get angle indexes
        AI = self.__angles[realIndex]['improperMap'] + self.__angles[realIndex]['otherMap']
        # append all mapped bonds to collector's random data
        self._atomsCollector._randomData = self._atomsCollector._randomData.union( set(AI) )
        # collect atom anglesIndexes
        self._atomsCollector.collect(realIndex, dataDict={'improperMap':self.__angles[realIndex]['improperMap'],
                                                          'otherMap'   :self.__angles[realIndex]['otherMap']})



    def plot(self, ax=None, nbins=50, subplots=True, split=None,
                   wspace=0.3, hspace=0.3,
                   histTtype='bar', lineWidth=None, lineColor=None,
                   xlabel=True, xlabelSize=16,
                   ylabel=True, ylabelSize=16,
                   legend=True, legendCols=1, legendLoc='best',
                   title=True, titleStdErr=True, titleAtRem=True,
                   titleUsedFrame=True, show=True):
        """
        Plot improper angles constraint distribution histogram.

        :Parameters:
            #. ax (None, matplotlib Axes): matplotlib Axes instance to plot in.
               If ax is given,  subplots parameters will be omitted.
               If None is given a new plot figure will be created.
            #. nbins (int): number of bins in histogram.
            #. subplots (boolean): Whether to add plot constraint on multiple
               axes.
            #. split (None, 'name', 'element'): To split plots into histogram
               per atom names, elements in addition to lower and upper bounds.
               If None is given, histograms will be built from lower and upper
               bounds only.
            #. wspace (float): The amount of width reserved for blank space
               between subplots, expressed as a fraction of the average axis
               width.
            #. hspace (float): The amount of height reserved for white space
               between subplots, expressed as a fraction of the average axis
               height.
            #. histTtype (string): the histogram type. optional among
               ['bar', 'barstacked', 'step', 'stepfilled']
            #. lineWidth (None, integer): bars contour line width.
               If None is given, then default value will be set automatically.
            #. lineColor (None, integer): bars contour line color.
               If None is given, then default value will be set automatically.
            #. xlabel (boolean): Whether to create x label.
            #. xlabelSize (number): The x label font size.
            #. ylabel (boolean): Whether to create y label.
            #. ylabelSize (number): The y label font size.
            #. legend (boolean): Whether to create the legend or not
            #. legendCols (integer): Legend number of columns.
            #. legendLoc (string): The legend location. Anything among
               'right', 'center left', 'upper right', 'lower right', 'best',
               'center', 'lower left', 'center right', 'upper left',
               'upper center', 'lower center' is accepted.
            #. title (boolean): Whether to create the title or not.
            #. titleStdErr (boolean): Whether to show constraint standard
               error value in title.
            #. titleAtRem (boolean): Whether to show engine's number of
               removed atoms.
            #. titleUsedFrame(boolean): Whether to show used frame name in
               title.
            #. show (boolean): Whether to render and show figure before
               returning.

        :Returns:
            #. figure (matplotlib Figure): matplotlib used figure.
            #. axes (matplotlib Axes, List): matplotlib axes or a list of axes.

        +------------------------------------------------------------------------------+
        |.. figure:: improper_angle_constraint_plot_method.png                         |
        |   :width: 530px                                                              |
        |   :height: 400px                                                             |
        |   :align: left                                                               |
        +------------------------------------------------------------------------------+
        """
        def _get_bins(dmin, dmax, boundaries, nbins):
            # create bins
            delta = float(dmax-dmin)/float(nbins-1)
            bins  = range(nbins)
            bins  = [b*delta for b in bins]
            bins  = [b+dmin for b in bins]
            # check boundaries
            bidx = 0
            for b in sorted(boundaries):
                for i in range(bidx, len(bins)-1):
                    bidx = i
                    # exact match with boundary
                    if b==bins[bidx]:
                        break
                    # boundary between two bins, move closest bin to boundary
                    if bins[bidx] < b < bins[bidx+1]:
                        if b-bins[bidx] > bins[bidx+1]-b:
                            bins[bidx+1] = b
                        else:
                            bins[bidx]   = b
                        break
            # return bins
            return bins
        # get constraint value
        output = self.get_constraint_value()
        if output is None:
            LOGGER.warn("%s constraint data are not computed."%(self.__class__.__name__))
            return
        # compute categories
        if split == 'name':
            splitV = self.engine.get_original_data("allNames")
        elif split == 'element':
            splitV = self.engine.get_original_data("allElements")
        else:
            splitV = None
        categories = {}
        atom2 = self.__anglesList[0]
        atom1 = self.__anglesList[1]
        atom3 = self.__anglesList[2]
        atom4 = self.__anglesList[3]
        lower = self.__anglesList[4]
        upper = self.__anglesList[5]
        for idx in xrange(self.__anglesList[0].shape[0]):
            if self._atomsCollector.is_collected(idx):
                continue
            if splitV is not None:
                a1 = splitV[ atom1[idx] ]
                a2 = splitV[ atom2[idx] ]
                a3 = splitV[ atom3[idx] ]
                a4 = splitV[ atom4[idx] ]
            else:
                a1 = a2 = a3 = a4 = ''
            l = lower[idx]
            u = upper[idx]
            k = (a1,a2,a3,a4,l,u)
            L = categories.get(k, [])
            L.append(idx)
            categories[k] = L
        ncategories = len(categories)
        # import matplotlib
        import matplotlib.pyplot as plt
        # get axes
        if ax is None:
            if subplots and ncategories>1:
                x = np.ceil(np.sqrt(ncategories))
                y = np.ceil(ncategories/x)
                FIG, N_AXES = plt.subplots(int(x), int(y) )
                N_AXES = N_AXES.flatten()
                FIG.subplots_adjust(wspace=wspace, hspace=hspace)
                [N_AXES[i].axis('off') for i in range(ncategories,len(N_AXES))]
            else:
                FIG  = plt.figure()
                AXES = FIG.gca()
                subplots = False
        else:
            AXES = ax
            FIG = AXES.get_figure()
            subplots = False
        # start plotting
        COLORS = ["b",'g','r','c','y','m']
        if subplots:
            for idx, key in enumerate(categories):
                a1,a2,a3,a4, L,U  = key
                L  = L*180./np.pi
                U  = U*180./np.pi
                LU = "(%.2f,%.2f)"%(L,U)
                label = "%s%s%s%s%s%s%s%s"%(a1,'-'*(len(a1)>0),a2,'-'*(len(a1)>0),a3,'-'*(len(a1)>0),a4,LU)
                COL  = COLORS[idx%len(COLORS)]
                AXES = N_AXES[idx]
                idxs = categories[key]
                data = self.data["angles"][idxs]*180./np.pi
                # get data limits
                mn = np.min(data)
                mx = np.max(data)
                # get bins
                BINS = _get_bins(dmin=mn, dmax=mx, boundaries=[L,U], nbins=nbins)
                # plot histogram
                D, _, P = AXES.hist(x=data, bins=BINS,
                                    color=COL, label=label,
                                    histtype=histTtype)
                # vertical lines
                Y = max(D)
                AXES.plot([L,L],[0,Y+0.1*Y], linewidth=1.0, color='k', linestyle='--')
                AXES.plot([U,U],[0,Y+0.1*Y], linewidth=1.0, color='k', linestyle='--')
                # legend
                if legend:
                    AXES.legend(frameon=False, ncol=legendCols, loc=legendLoc)
                # set axis labels
                if xlabel:
                    AXES.set_xlabel("$deg.$", size=xlabelSize)
                if ylabel:
                    AXES.set_ylabel("$number$"  , size=ylabelSize)
                if lineWidth is not None:
                    [p.set_linewidth(lineWidth) for p in P]
                if lineColor is not None:
                    [p.set_edgecolor(lineColor) for p in P]
                # update limits
                AXES.set_xmargin(0.1)
                AXES.autoscale()
        else:
            for idx, key in enumerate(categories):
                a1,a2,a3,a4, L,U  = key
                L  = L*180./np.pi
                U  = U*180./np.pi
                LU = "(%.2f,%.2f)"%(L,U)
                label = "%s%s%s%s%s%s%s%s"%(a1,'-'*(len(a1)>0),a2,'-'*(len(a1)>0),a3,'-'*(len(a1)>0),a4,LU)
                COL  = COLORS[idx%len(COLORS)]
                idxs = categories[key]
                data = self.data["angles"][idxs]*180./np.pi
                # get data limits
                mn = np.min(data)
                mx = np.max(data)
                # get bins
                BINS = _get_bins(dmin=mn, dmax=mx, boundaries=[L,U], nbins=nbins)
                # plot histogram
                D, _, P = AXES.hist(x=data, bins=BINS,
                                    color=COL, label=label,
                                    histtype=histTtype)
                # vertical lines
                Y = max(D)
                AXES.plot([L,L],[0,Y+0.1*Y], linewidth=1.0, color='k', linestyle='--')
                AXES.plot([U,U],[0,Y+0.1*Y], linewidth=1.0, color='k', linestyle='--')
                if lineWidth is not None:
                    [p.set_linewidth(lineWidth) for p in P]
                if lineColor is not None:
                    [p.set_edgecolor(lineColor) for p in P]
            # legend
            if legend:
                AXES.legend(frameon=False, ncol=legendCols, loc=legendLoc)
            # set axis labels
            if xlabel:
                AXES.set_xlabel("$deg.$", size=xlabelSize)
            if ylabel:
                AXES.set_ylabel("$number$"  , size=ylabelSize)
            # update limits
            AXES.set_xmargin(0.1)
            AXES.autoscale()

        # set title
        if title:
            FIG.canvas.set_window_title('Improper Angle Constraint')
            if titleUsedFrame:
                t = '$frame: %s$ : '%self.engine.usedFrame.replace('_','\\_')
            else:
                t = ''
            if titleAtRem:
                t += "$%i$ $rem.$ $at.$ - "%(len(self.engine._atomsCollector))
            if titleStdErr and self.standardError is not None:
                t += "$std$ $error=%.6f$ "%(self.standardError)
            if len(t):
                FIG.suptitle(t, fontsize=14)
        # set background color
        FIG.patch.set_facecolor('white')

        #show
        if show:
            plt.show()
        # return axes
        if subplots:
            return FIG, N_AXES
        else:
            return FIG, AXES

    def export(self, fname, delimiter='     ', comments='# ', split=None):
        """
        Export improper angles constraint distribution histogram.

        :Parameters:
            #. fname (path): full file name and path.
            #. delimiter (string): String or character separating columns.
            #. comments (string): String that will be prepended to the header.
            #. split (None, 'name', 'element'): To split output into per atom names,
               elements in addition to lower and upper bounds. If None output
               will be built from lower and upper bounds only.
        """
        # get constraint value
        output = self.get_constraint_value()
        if output is None:
            LOGGER.warn("%s constraint data are not computed."%(self.__class__.__name__))
            return
        # compute categories
        if split == 'name':
            splitV = self.engine.get_original_data("allNames")
        elif split == 'element':
            splitV = self.engine.get_original_data("allElements")
        else:
            splitV = None
        categories = {}
        atom2 = self.__anglesList[0]
        atom1 = self.__anglesList[1]
        atom3 = self.__anglesList[2]
        atom4 = self.__anglesList[3]
        lower = self.__anglesList[4]
        upper = self.__anglesList[5]
        for idx in xrange(self.__anglesList[0].shape[0]):
            if self._atomsCollector.is_collected(idx):
                continue
            if splitV is not None:
                a1 = splitV[ atom1[idx] ]
                a2 = splitV[ atom2[idx] ]
                a3 = splitV[ atom3[idx] ]
                a4 = splitV[ atom4[idx] ]
            else:
                a1 = a2 = a3 = a4 = ''
            l = lower[idx]
            u = upper[idx]
            k = (a1,a2,a3,a4,l,u)
            L = categories.get(k, [])
            L.append(idx)
            categories[k] = L
        ncategories = len(categories)
        # create data
        for idx, key in enumerate(categories):
            idxs = categories[key]
            data = self.data["angles"][idxs]
            categories[key] = [str(d) for d in data]
        # adjust data size
        maxSize = max( [len(v) for v in categories.values()] )
        for key in categories:
            data = categories[key]
            add  = maxSize-len(data)
            if add > 0:
                categories[key] = data + ['']*add
        # start creating header and data
        sortCa = sorted( categories )
        header = []
        for key in sortCa:
            a1,a2,a3,a4, L,U   = key
            L  = L*180./np.pi
            U  = U*180./np.pi
            LU = "(%.2f,%.2f)"%(L,U)
            header.append( ("%s%s%s%s%s%s%s%s"%(a1,'-'*(len(a1)>0),a2,'-'*(len(a1)>0),a3,'-'*(len(a1)>0),a4,LU)).replace(' ','') )
        data   = [categories[key] for key in sortCa]
        # save
        data = np.transpose(data)
        np.savetxt(fname     = fname,
                   X         = data,
                   fmt       = '%s',
                   delimiter = delimiter,
                   header    = " ".join(header),
                   comments  = comments)



"""
Collection of methods and classes definition useful for constraints computation
"""
# standard libraries imports
from __future__ import print_function

# external libraries imports
import numpy as np

# fullrmc library imports
from ..Globals import LOGGER, FLOAT_TYPE, PI
from ..Globals import str, long, unicode, bytes, basestring, range, xrange, maxint
from ..Core.Collection import is_number, get_caller_frames

class ShapeFunction(object):
    """
    Shape function used to correct for particle shape. The shape
    function is subtracted from the total G(r) of g(r). It must
    be used when non-periodic boundary conditions are used to take
    into account the atomic density drop and to correct for the
    :math:`\\rho_{0}` approximation.

    :Parameters:
        #. engine (Engine): The fitting engine.
        #. weighting (string): The elements weighting.
        #. qmin (number): The minimum reciprocal distance q in
           :math:`\\AA^{-1}` considered to compute the shape function.
        #. qmax (number): The maximum reciprocal distance q in
           :math:`\\AA^{-1}` considered to compute the shape function.
        #. dq (number): The reciprocal distance bin size in
           :math:`\\AA^{-1}` considered to compute the shape function.
        #. rmin (number): The minimum distance in :math:`\\AA` considered
           upon building the histogram prior to computing the shape function.
        #. rmax (number): The maximum distance in :math:`\\AA`
           considered upon building the histogram prior to computing the shape
           function.
        #. dr (number): The bin size in :math:`\\AA` considered upon
           building the histogram prior to computing the shape function.

    **N.B: tweak qmax as small as possible to reduce the wriggles ...**
    """
    def __init__(self, engine, weighting="atomicNumber",
                       qmin=0.001, qmax=1, dq=0.005,
                       rmin=0.00, rmax=100, dr=1):
        # get qmin
        assert is_number(qmin), LOGGER.error("qmin must be a number")
        qmin = FLOAT_TYPE(qmin)
        assert qmin>0, LOGGER.error("qmin '%s' must be bigger than 0"%qmin)
        # get qmin
        assert is_number(qmax), LOGGER.error("qmax must be a number")
        qmax = FLOAT_TYPE(qmax)
        assert qmax>qmin, LOGGER.error("qmax '%s' must be bigger than qmin '%s'"%(qmin,qmax))
        # get dq
        assert is_number(dq), LOGGER.error("dq must be a number")
        dq = FLOAT_TYPE(dq)
        assert dq>0, LOGGER.error("dq '%s' must be bigger than 0"%dq)
        # import StructureFactorConstraint
        from fullrmc.Constraints.StructureFactorConstraints import StructureFactorConstraint
        # overload constraint
        class _ShapeFunctionStructureFactor(StructureFactorConstraint):
            """This overloading is needed to avoid the constraint being
            saved upon setting the different properties"""
            def _get_repository(self, *args, **kwargs):
                return None
            def _dump_to_repository(self, *args, **kwargs):
                return
        # create StructureFactorConstraint
        Q = np.arange(qmin, qmax, dq)
        D = np.transpose([Q, np.zeros(len(Q))]).astype(FLOAT_TYPE)
        #self._SFC = StructureFactorConstraint(rmin=rmin, rmax=rmax, dr=dr, experimentalData=D, weighting="atomicNumber")
        self._SFC = _ShapeFunctionStructureFactor(rmin=rmin, rmax=rmax, dr=dr, experimentalData=D, weighting="atomicNumber")
        self._SFC._set_engine(engine)
        self._SFC.listen(message="engine set")
        # set parameters
        self._rmin = FLOAT_TYPE(rmin)
        self._rmax = FLOAT_TYPE(rmax)
        self._dr   = FLOAT_TYPE(dr)
        self._qmin = FLOAT_TYPE(qmin)
        self._qmax = FLOAT_TYPE(qmax)
        self._dq   = FLOAT_TYPE(dq)
        self._weighting = weighting

    def __get_Gr_from_Sq(self, qValues, rValues, Sq):
        Gr    = np.zeros(len(rValues), dtype=FLOAT_TYPE)
        sq_1  = Sq-1
        qsq_1 = qValues*sq_1
        dq = qValues[1]-qValues[0]
        for ridx, r in enumerate(rValues):
            sinqr_dq = dq * np.sin(qValues*r)
            Gr[ridx] = (2./PI) * np.sum( qsq_1 * sinqr_dq )
        return Gr

    def get_Gr_shape_function(self, rValues, compute=True):
        """
        Get shape function of G(r) used in a PairDistributionConstraint.

        :Parameters:
            #. rValues (numpy.ndarray): The r values array.
            #. compute (boolean): whether to recompute shape
               function reciprocal data.

        :Returns:
            #. shapeFunction (numpy.ndarray): The compute shape function.
        """
        # compute data
        if compute: self._SFC.compute_data()
        # get shape function
        return self.__get_Gr_from_Sq( qValues=self._SFC.experimentalQValues,
                                      rValues=rValues,
                                      Sq=self._SFC.get_constraint_value()['total'])

    def get_gr_shape_function(self, rValues, compute=True):
        """
        Get shape function of g(r) used in a PairCorrelationConstraint.

        :Parameters:
            #. rValues (numpy.ndarray): The r values array.
            #. compute (boolean): whether to recompute shape
               function reciprocal data.

        :Returns:
            #. shapeFunction (numpy.ndarray): The compute shape function.
        """
        sFunc = self.get_Gr_shape_function(rValues=rValues, compute=compute)
        rho0  = self.engine.numberDensity #(self._SFC.engine.numberOfAtoms/self._SFC.engine.volume).astype(FLOAT_TYPE)
        return sFunc / (FLOAT_TYPE(4.)*PI*rho0*rValues)
